#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>

#include <sys/signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

static void open_tty(const char * tty)
{
  struct sigaction sa;
  int fd;

  /* Set up new standard input. */

  if(chown(tty, 0, 0) || chmod(tty, 0600))
     exit(0);

  sa.sa_handler = SIG_IGN;
  sa.sa_flags = 0;

  sigemptyset(&sa.sa_mask);
  sigaction(SIGHUP, &sa, NULL);

  if((fd = open(tty, O_RDWR, 0)) < 0 || ioctl(fd, TIOCSCTTY, (void *)1) == -1)
     exit(0);

  if(!isatty(fd))
     exit(0);

  vhangup();

  /* Get rid of the present stdout/stderr. */

  close(2);
  close(1);
  close(0);
  close(fd);

  if(open(tty, O_RDWR, 0) != 0)
     exit(0);

  /* Set up standard output and standard error file descriptors. */

  if(dup(0) != 1 || dup(0) != 2)
     exit(0);

  sa.sa_handler = SIG_DFL;
  sa.sa_flags = 0;

  sigemptyset(&sa.sa_mask);
  sigaction(SIGHUP, &sa, NULL);
}

static const char * settings[] = {
 "TERM=linux",
 "USER=root",
 "PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin:/usr/local/sbin",
 NULL
};

/* usage: simgetty ttyX [login-program] */ 

int main(int argc, char **argv)
{
  char tty[1024];
  
  char * login = "/bin/login";

  int i;

  if(NULL == argv[1])      
    exit(0);

  for(i = 0 ; ; i++)
   {
     if(NULL == settings[i]) /* end of list */
        break ;
	
     if(0 != putenv(settings[i]))
       printf("environment seems to be full!\n");
   }
  
  sprintf(tty, "/dev/%s", argv[1]);
  
  open_tty(tty);

  ioctl(0, TCFLSH, 2);

  sleep(1); 
  
  setuid(0);
  seteuid(0);
  
  if(argv[2])
    login = argv[2];
    
  execl(login, login, NULL);

  sleep(1); 

  return 0 ;
}
